#!/usr/bin/env python3

# This value is not the one controlling the version in the (GitLab) pipeline
# This value is only necessary for automated pickup, and for manual deployments to PyPI
version = '0.3.0'



#!/usr/bin/env python3

# Imports of module(s) internal to this project/package
from blackjack.cli import cli  # pragma: no cover


if __name__ == '__main__':
    # Just pass on to the cli-function in the
    # 'blackjack/cli.py' file, and let it be the 'main()' function.
    cli()



#!/usr/bin/env python3


def greeting(
    who_you_want_to_greet: str = None,
) -> str:
    who_you_want_to_greet = who_you_want_to_greet.strip()
    if (
        not isinstance(who_you_want_to_greet, str)
        or not(who_you_want_to_greet)
        or any(
            # If any of these remain in string
            # after strip()-ing the string...
            char in who_you_want_to_greet
            for char in ('\r', '\n')
        )
    ):
        who_you_want_to_greet = 'you'
    return f"Welcome to a game of blackjack, {who_you_want_to_greet}!"



#!/usr/bin/env python3

# Python standard library imports
import os
import typing

# Non-standard library python package imports
import click
import requests

# Imports of module(s) internal to this project/package
from blackjack import version
from blackjack.game import (
    Card,
    CardSuit,
    CardValue,
)
from blackjack.utils import greeting


NICKNAME_ENV_VAR = 'BLACKJACK_NICKNAME'


@click.command(
    context_settings=dict(
        help_option_names=['-h', '--help']
    )
)
@click.option(
    f"--{NICKNAME_ENV_VAR.replace('_', '-').lower()}", '-n', 'nickname',
    default=os.environ.get(NICKNAME_ENV_VAR, 'you'),
    type=str, required=True, hide_input=False, show_default=True,
    help=(
        'Your nickname in this blackjack game.'
        f"Can also be specified through environment variable '{NICKNAME_ENV_VAR}'."
    ),
)
@click.option(
    f"--shuffle-url", '-u', 'shuffle_url',
    default='http://nav-deckofcards.herokuapp.com/shuffle',
    type=str, required=True, hide_input=False, show_default=True,
    help=(
        'The URL to which this application will perform a HTTP GET method, '
        'expecting a JSON array of elements á la; '
        '`[...,{"suit":"CLUBS","value":"K"},{"suit":"SPADES","value":"8"},`'
    ),
)
@click.version_option(version=version)
@click.pass_context
def cli(
    ctx: typing.Any,
    nickname: str,
    shuffle_url: str,
):
    click.echo(greeting(nickname))
    shuffled_cards_json = requests.get(shuffle_url).json()
    shuffled_cards = [
        Card(
            value=CardValue[card_json['value']],
            suit=CardSuit[card_json['suit']],
        )
        for card_json
        in shuffled_cards_json
    ]
    click.echo(f"Here are the cards fetched from '{shuffle_url}' (in order);\n[")
    for card in shuffled_cards:
        click.echo(f"\t{str(card).title()},")
    click.echo(']')



#!/usr/bin/env python3
"""
Sub-module defining a `Deck`, a `Card`, a card's `Suit` and a card's `Value`.
"""

# Python standard library imports
import dataclasses
import enum
import itertools
import json
import textwrap
import typing


CardValue = enum.Enum(
    value='CardValue',
    names=itertools.chain.from_iterable(
        itertools.product(v, (k,))
        for k, v
        in {
            enum.auto(): ('Two', '2',),
            enum.auto(): ('Three', '3',),
            enum.auto(): ('Four', '4',),
            enum.auto(): ('Five', '5',),
            enum.auto(): ('Six', '6',),
            enum.auto(): ('Seven', '7',),
            enum.auto(): ('Eight', '8',),
            enum.auto(): ('Nine', '9',),
            enum.auto(): ('Ten', '10',),
            enum.auto(): ('Jack', 'J',),
            enum.auto(): ('Queen', 'Q',),
            enum.auto(): ('King', 'K',),
            enum.auto(): ('Ace', 'A',),
        }.items()
    ),
)
CardSuit = enum.Enum(
    value='CardSuit',
    names=itertools.chain.from_iterable(
        itertools.product(v, (k,))
        for k, v
        in {
            enum.auto(): ('HEARTS', 'H',),
            enum.auto(): ('DIAMONDS', 'D',),
            enum.auto(): ('SPADES', 'S',),
            enum.auto(): ('CLUBS', 'C',),
        }.items()
    ),
)


@dataclasses.dataclass(
    frozen=True,
    eq=True,
)
class Card:
    value: CardValue
    suit: CardSuit

    @property
    def points(self) -> int:
        map_of_points_per_value: typing.Dict[str, int] = {
            CardValue[str(v)]: v for v in range(2, 11)
        }
        map_of_points_per_value.update(
            {
                CardValue['J']: 10,
                CardValue['Q']: 10,
                CardValue['K']: 10,
                CardValue['A']: 11,
            }
        )
        return map_of_points_per_value[self.value]

    def __str__(self) -> str:
        return f"{self.value.name} of {self.suit.name}"

    def __repr__(self) -> str:
        return textwrap.dedent(
            # The next two lines are crucial for `dedent()` to work.
            f"""\
            Card(
                value=CardValue['{self.value.name}'],
                suit=CardSuit['{self.suit.name}']
            )"""
        )

    def __eq__(self, other) -> bool:
        return (
            self.value == other.value
            and self.suit == self.suit
        )

    def __lt__(self, other) -> bool:
        # The test 'ensure_all_card_values_are_sortable'
        #   tests this property... Justified '#pragma: no cover'
        return self.points < other.points   # pragma: no cover


class CardJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Card):
            return dict(
                suit=obj.suit.name,
                value=obj.value.name,
            )
        # We don't care about testing the library-standard json
        # serialization behaviour...
        return json.JSONEncoder.default(self, obj)  # pragma: no cover



#!/usr/bin/env python3.7

# Python standard library imports
from pathlib import Path

# Non-standard library python package imports
from setuptools import (  # type: ignore
    find_packages,
    setup,
)

# Imports of module(s) internal to this project/package
from blackjack import version


setup(
    # Metadata
    name='x10-blackjack',
    version=version,
    author_email='x10an14@gmail.com',
    description='A small blackjack game.',
    long_description=Path('README.md').read_text(),
    long_description_content_type='text/markdown',
    url='https://gitlab.com/x10an14/blackjack',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Operating System :: OS Independent',
        'Topic :: Games/Entertainment',
        'Topic :: Software Development',
    ],

    # Package data
    packages=find_packages(),
    include_package_data=True,
    install_requires=list(),
    entry_points='''
        [console_scripts]
        blackjack=blackjack.cli:cli
    ''',
    python_requires='>=3.7'
)
